package org.unicore.outcome;

import org.unicore.Unicore;

/**
 * The allowed status values of an AbstractAction
 * on an NJS.
 * <p>
 * AbstractActionStatus's are arranged in a hierarchy to 
 * allow testing at the appropriate level of detail to the
 * reason for the test.
 * <p>
 * All Statuses are either DONE or NOT_DONE
 * <p>
 * All DONE Statuses are either SUCCESSFUL, NOT_SUCCESSFUL, NEVER_RUN or NEVER_TAKEN
 * <p>
 * It is possible to describe the status of an AbstractAction in
 * more detail but is not required that an NJS use these values.
 * <p>
 * AbstractActions that are NOT_DONE can be either CONSIGNED, PENDING, READY, HELD or EXECUTING.
 * <p>
 * EXECUTING AbstractActions can be either QUEUED, RUNNING or SUSPENDED.
 * <p>
 * Valid NOT_SUCCESSFUL status values are FAILED_IN_EXECUTION, KILLED,
 * FAILED_IN_INCARNATION, and FAILED_IN_CONSIGN.
 * 
 * @see Outcome
 *
 * @author  Sven van den Berghe (fecit) 
 * 
 * @since AJO 2
 *
 * @version AbstractActionStatus.java 1.6 09/09/02 
 **/
public class AbstractActionStatus extends Unicore {

    static final long serialVersionUID = -1036551147605749870L;

    protected int level1;
    protected int level2;
    protected int level3;

    private String info;

    protected AbstractActionStatus(String info, int level1) {
        this.info     = info;
        this.level1   = level1;
        this.level2   = 0;
        this.level3   = 0;
    }

    protected AbstractActionStatus(String info, int level1, int level2) {
        this.info     = info;
        this.level1   = level1;
        this.level2   = level2;
        this.level3   = 0;
    }

    protected AbstractActionStatus(String info, int level1, int level2, int level3) {
        this.info     = info;
        this.level1   = level1;
        this.level2   = level2;
        this.level3   = level3;
    }

    public String toString() {return info;}

    /**
     * Test for equality. 
     * <p>
     * This test returns equality across different Java Virtual Machines.
     *
     **/
    public boolean equals(Object in) {
        if( in.getClass() == this.getClass() ) {
            AbstractActionStatus inn = (AbstractActionStatus) in;
            return this.level1 == inn.level1 &&
                   this.level2 == inn.level2 &&
                   this.level3 == inn.level3;
        }
        else {
            return false;
        }
    }

    /**
     * Return a hash code based on the values used in the equality test.
     *
     **/
    public int hashCode() {
        return level1 | level2 | level3;
    }

    /**
     * Return true if the input is equivalent to this. 
     * <p>
     * Equivalent means that they have the same value at the most refined
     * common level of definition e.g.
     * <p>
     *      FAILED_IN_EXECUTION is equivalent to NOT_SUCCESSFUL because it
     *      is a refinement of NOT_SUCCESSFUL <br>
     *
     *      FAILED_IN_EXECUTION is also equivalent to DONE  because it is a 
     *      refinement of DONE <br>
     *
     *      FAILED_IN_CONSIGN is equivalent to NOT_SUCCESSFUL  because
     *      it is a refinement of NOT_SUCCESSFUL <br>
     *
     *      <em>but</em> FAILED_IN_EXECUTION is not equivalent to FAILED_IN_CONSIGN 
     *      because they are different refinements of NOT_SUCCESSFUL <br>
     *
     *      FAILED_IN_CONSIGN is not equivalent to SUCCESSFUL because it
     *      is a refinement of NOT_SUCCESSFUL (not a refinement of SUCCESSFUL) <br>
     *
     *      NOT_SUCCESSFUL and SUCCESSFUL are both equivalent to DONE but
     *      not to each other etc 
     *
     **/
    public boolean isEquivalent(AbstractActionStatus inn) {
        if(this.level1 == inn.level1) {
            if((this.level2 == inn.level2) || (this.level2==0) || (inn.level2 ==0)) {
                if((this.level3 == inn.level3) || (this.level3==0) || (inn.level3 ==0)) {
                    return true;
                }
                else {
                    return false;
                }
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }


    /**
     * The AbstractAction has completed execution.
     * <p>
     * This a least refined level of status (along with NOT_DONE).
     **/
    public static final AbstractActionStatus DONE    = new AbstractActionStatus("DONE",1);

        /**
         * The execution of the AbstractAction completed successfully.
         * <p>
         * This is a refinement of the DONE status.
         * <p>
         * This is at the same level of 
         * refinement as NOT_SUCCESSFUL, NEVER_RUN and NEVER_TAKEN.
         * <p>
         * 
         **/
        public static final AbstractActionStatus SUCCESSFUL = new  AbstractActionStatus("SUCCESSFUL", 1, 1);

        /**
         * The execution of the AbstractAction failed.
         * <p>
         * This is a refinement of the DONE status.
         * <p>
         * This is at the same level of 
         * refinement as SUCCESSFUL, NEVER_RUN and NEVER_TAKEN.
         * <p>
         * 
         **/
        public static final AbstractActionStatus NOT_SUCCESSFUL  = new  AbstractActionStatus("NOT_SUCCESSFUL", 1, 2);

            /**
             * The AbstractAction failed in execution or a child AbstractAction
             * of an AJO finished with a status equivalent to NOT_SUCCESSFUL.
             * <p>
             * This is a refinement of the DONE status.
             * <p>
             * This is a refinement of the NOT_SUCCESSFUL status.
             * <p>
             * This is at the same level of refinement as 
             * FAILED_IN_CONSIGN, FAILED_IN_INCARNATION and KILLED
             *
             **/
            public static final AbstractActionStatus FAILED_IN_EXECUTION   
                            = new  AbstractActionStatus("FAILED_IN_EXECUTION", 1, 2, 1);

            /**
             * The AbstractAction failed in consign (to another NJS or a BSS).
             * <p>
             * This is a refinement of the DONE status.
             * <p>
             * This is a refinement of the NOT_SUCCESSFUL status.
             * <p>
             * This is at the same level of refinement as 
             * FAILED_IN_EXECUTION, FAILED_IN_INCARNATION and KILLED
             *
             **/
            public static final AbstractActionStatus FAILED_IN_CONSIGN     
                            = new  AbstractActionStatus("FAILED_IN_CONSIGN", 1, 2, 2);

            /**
             * The AbstractAction failed during incarnation.
             * <p>
             * This is a refinement of the DONE status.
             * <p>
             * This is a refinement of the NOT_SUCCESSFUL status.
             * <p>
             * This is at the same level of refinement as 
             * FAILED_IN_EXECUTION, FAILED_IN_CONSIGN and KILLED
             *
             **/
            public static final AbstractActionStatus FAILED_IN_INCARNATION 
                            = new  AbstractActionStatus("FAILED_IN_INCARNATION", 1, 2, 3);

            /**
             * The AbstractAction was killed by the user or an administrator.
             * <p>
             * This is a refinement of the DONE status.
             * <p>
             * This is a refinement of the NOT_SUCCESSFUL status.
             * <p>
             * This is at the same level of refinement as 
             * FAILED_IN_EXECUTION FAILED_IN_CONSIGN and FAILED_IN_INCARNATION
             *
             **/
            public static final AbstractActionStatus KILLED         = new  AbstractActionStatus("KILLED", 1, 2, 4);

        /**
         * The AbstractAction was never executed because one of the Actions on which
         * it depended finished execution with a status equivalent to NOT_SUCCESSFUL
         * or NEVER_RUN.
         * <p>
         * This is a refinement of the DONE status.
         * <p>
         * This is at the same level of 
         * refinement as SUCCESSFUL, NOT_SUCCESSFUL and NEVER_TAKEN.
         * <p>
         * 
         **/
        public static final AbstractActionStatus NEVER_RUN = new  AbstractActionStatus("NEVER_RUN", 1, 3);

        /**
         * The Action was never executed because the branch on which it lies
         * was not selected for execution by a flow control task.
         * <p>
         * This is a refinement of the DONE status.
         * <p>
         * This is at the same level of 
         * refinement as SUCCESSFUL, NOT_SUCCESSFUL and NEVER_RUN.
         * <p>
         * 
         **/
        public static final AbstractActionStatus NEVER_TAKEN = new  AbstractActionStatus("NEVER_TAKEN", 1, 4);

    /**
     * The AbstractAction has not completed execution.
     * <p>
     * AbstractActions with this status are either executing or waiting
     * to start executing (because, for example, not all predecessor AbstractActions
     * have a DONE status).
     * <p>
     * This a least refined level of status (along with DONE).
     *
     **/
    public static final AbstractActionStatus NOT_DONE = new AbstractActionStatus("NOT_DONE", 2);

        /**
         * The AbstractAction is consigned to a NJS or a BSS for execution.
         * <p>
         * This is a refinement of the NOT_DONE status.
         * <p>
         * This is at the same level of refinement as 
         * PENDING, READY, HELD and EXECUTING.
         *
         **/
        public static final AbstractActionStatus CONSIGNED = new  AbstractActionStatus("CONSIGNED", 2, 1);

        /**
         * The AbstractAction is waiting for dependancies to become free
         * before starting to execute.
         * <p>
         * This is a refinement of the NOT_DONE status.
         * <p>
         * This is at the same level of refinement as 
         * CONSIGNED, READY, HELD and EXECUTING.
         *
         **/
        public static final AbstractActionStatus PENDING   = new  AbstractActionStatus("PENDING", 2, 2);

        /**
         * The AbstractAction's dependancies are free but it has not started
         * executing.
         * <p>
         * This is a refinement of the NOT_DONE status.
         * <p>
         * This is at the same level of refinement as 
         * CONSIGNED, PENDING, HELD and EXECUTING.
         *
         **/
        public static final AbstractActionStatus READY   = new  AbstractActionStatus("READY", 2, 3);

        /**
         * The AbstractAction is held by user or administrator.
         * <p>
         * This is a refinement of the NOT_DONE status.
         * <p>
         * This is at the same level of refinement as 
         * CONSIGNED, PENDING, READY and EXECUTING
         *
         **/
        public static final AbstractActionStatus HELD      = new  AbstractActionStatus("HELD", 2, 4);

        /**
         * The Action is executing.
         * <p>
         * This is a refinement of the NOT_DONE status.
         * <p>
         * This is at the same level of refinement as 
         * CONSIGNED, PENDING, READY and HELD
         *
         **/
        public static final AbstractActionStatus EXECUTING = new  AbstractActionStatus("EXECUTING", 2, 5);

            /**
             * The AbstractAction is consigned to a BSS and is queued there.
             * <p>
             * This is a refinement of the NOT_DONE status.
             * <p>
             * This is a refinement of the EXECUTING status.
             * <p>
             * This is at the same level of refinement as 
             * RUNNING and SUSPENDED.
             *
             **/
            public static final AbstractActionStatus QUEUED    = new  AbstractActionStatus("QUEUED", 2, 5, 1);

            /**
             * The AbstractAction is consigned to a BSS and is running there.
             * <p>
             * This is a refinement of the NOT_DONE status.
             * <p>
             * This is a refinement of the EXECUTING status.
             * <p>
             * This is at the same level of refinement as 
             * QUEUED and SUSPENDED.
             *
             **/
            public static final AbstractActionStatus RUNNING   = new  AbstractActionStatus("RUNNING", 2, 5, 2);

            /**
             * The AbstractAction is consigned to a BSS and is held there by user or administrator.
             * <p>
             * This is a refinement of the NOT_DONE status.
             * <p>
             * This is a refinement of the EXECUTING status.
             * <p>
             * This is at the same level of refinement as 
             * RUNNING and QUEUED.
             *
             **/
            public static final AbstractActionStatus SUSPENDED = new  AbstractActionStatus("SUSPENDED", 2, 5, 3);
            
}
//
// Change history:
//            5 May 2000 SvdB V3.0, added NEVER_TAKEN 
//            7 May 1999 SvdB SvdB removed redundant line 
//           21 Dec 1998 SvdB SvdB Repaired coding 
//           10 Dec 1998 SvdB More accurate string messages 
//            7 Dec 1998 SvdB Second release to Unicore 
//           10 Nov 1998 SvdB Created 
//
//             Copyright (c) UNICORE Forum e.V. 2000 - 2002
//
//     Use and distribution is subject to the Community Source Licence.
//     A copy of the Community Source Licence came with this distribution.
//
//     Further information can be obtained from www.unicore.org
